@@ -1341,6 +1341,18 @@ class Trimesh(Geometry3D):
         self._cache['face_adjacency_edges'] = edges
         return adjacency
 
+    @caching.cache_decorator
+    def face_neighborhood(self):
+        """
+        Find faces that share a vertex i.e. 'neighbors' faces.
+
+        Returns
+        ----------
+        neighborhood : (n, 2) int
+          Pairs of faces which share a vertex
+        """
+        return graph.face_neighborhood(self)
+
     @caching.cache_decorator
     def face_adjacency_edges(self):
         """
@@ -1840,9 +1852,10 @@ class Trimesh(Geometry3D):
         cost : float
           Average square distance per point
         """
-        mesh_to_other, cost = registration.mesh_other(mesh=self,
-                                                      other=other,
-                                                      **kwargs)
+        mesh_to_other, cost = registration.mesh_other(
+            mesh=self,
+            other=other,
+            **kwargs)
         return mesh_to_other, cost
 
     def compute_stable_poses(self,
