@@ -4,7 +4,6 @@ API wrapped to match our native raytracer.
 """
 import numpy as np
 
-from collections import deque
 from copy import deepcopy
 
 from pyembree import __version__ as _ver
@@ -19,6 +18,8 @@ from .. import util
 from .. import caching
 from .. import intersections
 
+from ..constants import log_time
+
 # the factor of geometry.scale to offset a ray from a triangle
 # to reliably not hit its origin triangle
 _ray_offset_factor = 1e-4
@@ -41,12 +42,12 @@ class RayMeshIntersector(object):
 
         Parameters
         -------------
-        geometry     : Trimesh object
-                         Mesh to do ray tests on
+        geometry : Trimesh object
+          Mesh to do ray tests on
         scale_to_box : bool
-                         If true, will scale mesh to approximate
-                         unit cube to avoid problems with extreme
-                         large or small meshes.
+          If true, will scale mesh to approximate
+          unit cube to avoid problems with extreme
+          large or small meshes.
         """
         self.mesh = geometry
         self._scale_to_box = scale_to_box
@@ -83,15 +84,19 @@ class RayMeshIntersector(object):
 
         Parameters
         ----------
-        ray_origins:    (n,3) float, origins of rays
-        ray_directions: (n,3) float, direction (vector) of rays
-
+        ray_origins : (n, 3) float
+          Origins of rays
+        ray_directions : (n, 3) float
+          Direction (vector) of rays
 
         Returns
         ---------
-        locations: (n) sequence of (m,3) intersection points
-        index_ray: (n,) int, list of ray index
-        index_tri: (n,) int, list of triangle (face) indexes
+        locations : (m) sequence of (p, 3) float
+          Intersection points
+        index_ray : (m,) int
+          Indexes of ray
+        index_tri : (m,) int
+          Indexes of mesh.faces
         """
         (index_tri,
          index_ray,
@@ -103,6 +108,7 @@ class RayMeshIntersector(object):
 
         return locations, index_ray, index_tri
 
+    @log_time
     def intersects_id(self,
                       ray_origins,
                       ray_directions,
@@ -113,36 +119,48 @@ class RayMeshIntersector(object):
         Find the triangles hit by a list of rays, including
         optionally multiple hits along a single ray.
 
+
         Parameters
         ----------
-        ray_origins:      (n,3) float, origins of rays
-        ray_directions:   (n,3) float, direction (vector) of rays
-        multiple_hits:    bool, if True will return every hit along the ray
-                                if False will only return first hit
-        return_locations: bool, should we return hit locations or not
+        ray_origins : (n, 3) float
+          Origins of rays
+        ray_directions : (n, 3) float
+          Direction (vector) of rays
+        multiple_hits : bool
+          If True will return every hit along the ray
+          If False will only return first hit
+        max_hits : int
+          Maximum number of hits per ray
+        return_locations : bool
+          Should we return hit locations or not
 
         Returns
-        ----------
-        index_tri: (m,) int, index of triangle the ray hit
-        index_ray: (m,) int, index of ray
-        locations: (m,3) float, locations in space
+        ---------
+        index_tri : (m,) int
+          Indexes of mesh.faces
+        index_ray : (m,) int
+          Indexes of ray
+        locations : (m) sequence of (p, 3) float
+          Intersection points, only returned if return_locations
         """
         # make sure input is _dtype for embree
-        ray_origins = np.asanyarray(
+        ray_origins = np.array(
             deepcopy(ray_origins),
             dtype=np.float64)
         ray_directions = np.asanyarray(ray_directions,
                                        dtype=np.float64)
+        if ray_origins.shape != ray_directions.shape:
+            raise ValueError('Ray origin and direction don\'t match!')
         ray_directions = util.unitize(ray_directions)
 
         # since we are constructing all hits, save them to a deque then
         # stack into (depth, len(rays)) at the end
-        result_triangle = deque()
-        result_ray_idx = deque()
-        result_locations = deque()
+        result_triangle = []
+        result_ray_idx = []
+        result_locations = []
 
         # the mask for which rays are still active
-        current = np.ones(len(ray_origins), dtype=np.bool)
+        current = np.ones(len(ray_origins), dtype=bool)
 
         if multiple_hits or return_locations:
             # how much to offset ray to transport to the other side of face
@@ -162,10 +180,10 @@ class RayMeshIntersector(object):
             # run the pyembree query
             # if you set output=1 it will calculate distance along
             # ray, which is bizzarely slower than our calculation
+
             query = self._scene.run(
                 ray_origins[current],
                 ray_directions[current])
-
             # basically we need to reduce the rays to the ones that hit
             # something
             hit = query != -1
@@ -222,11 +240,14 @@ class RayMeshIntersector(object):
         index_ray = np.hstack(result_ray_idx)
 
         if return_locations:
-            locations = np.array(result_locations)
+            locations = (
+                np.zeros((0, 3), float) if len(result_locations) == 0
+                else np.array(result_locations))
 
             return index_tri, index_ray, locations
         return index_tri, index_ray
 
+    @log_time
     def intersects_first(self,
                          ray_origins,
                          ray_directions):
@@ -236,12 +257,15 @@ class RayMeshIntersector(object):
 
         Parameters
         ----------
-        ray_origins:    (n,3) float, origins of rays
-        ray_directions: (n,3) float, direction (vector) of rays
+        ray_origins : (n, 3) float
+          Origins of rays
+        ray_directions : (n, 3) float
+          Direction (vector) of rays
 
         Returns
         ----------
-        triangle_index: (n,) int, index of triangle ray hit, or -1 if not hit
+        triangle_index : (n,) int
+          Index of triangle ray hit, or -1 if not hit
         """
 
         ray_origins = np.asanyarray(deepcopy(ray_origins))
@@ -259,13 +283,16 @@ class RayMeshIntersector(object):
 
 
         Parameters
-        ----------
-        ray_origins:    (n,3) float, origins of rays
-        ray_directions: (n,3) float, direction (vector) of rays
+        -----------
+        ray_origins : (n, 3) float
+          Origins of rays
+        ray_directions : (n, 3) float
+          Direction (vector) of rays
 
         Returns
         ----------
-        hit:            (n,) bool, did each ray hit the surface
+        hit : (n,) bool
+          Did each ray hit the surface
         """
 
         first = self.intersects_first(ray_origins=ray_origins,
@@ -281,7 +308,7 @@ class RayMeshIntersector(object):
 
         Parameters
         ---------
-        points: (n,3) points in space
+        points: (n, 3) points in space
 
         Returns
         ---------
@@ -299,8 +326,8 @@ class _EmbreeWrap(object):
     """
 
     def __init__(self, vertices, faces, scale):
-        scaled = np.asanyarray(vertices,
-                               dtype=np.float64)
+        scaled = np.array(vertices,
+                          dtype=np.float64)
         self.origin = scaled.min(axis=0)
         self.scale = float(scale)
         scaled = (scaled - self.origin) * self.scale
@@ -310,11 +337,11 @@ class _EmbreeWrap(object):
         TriangleMesh(
             scene=self.scene,
             vertices=scaled.astype(_embree_dtype),
-            indices=faces.astype(np.int32))
+            indices=faces.view(np.ndarray).astype(np.int32))
 
     def run(self, origins, normals, **kwargs):
-        scaled = (np.asanyarray(origins,
-                                dtype=np.float64) - self.origin) * self.scale
+        scaled = (np.array(origins,
+                           dtype=np.float64) - self.origin) * self.scale
 
         return self.scene.run(scaled.astype(_embree_dtype),
                               normals.astype(_embree_dtype),
