@@ -1,9 +1,10 @@
 import uuid
-import copy
 
 import numpy as np
 import collections
 
+from copy import deepcopy
+
 from .. import util
 from .. import caching
 from .. import transformations
@@ -167,7 +168,11 @@ class SceneGraph(object):
         copied : TransformForest
           Copy of current object.
         """
-        return copy.deepcopy(self)
+        # create a copy without transferring cache
+        copied = SceneGraph()
+        copied.base_frame = deepcopy(self.base_frame)
+        copied.transforms = deepcopy(self.transforms)
+        return copied
 
     def to_flattened(self):
         """
@@ -637,7 +642,7 @@ class EnforcedForest(object):
                 self._cache[(u, v)] = backward
                 return backward
             elif forward[-1] is None and backward[-1] is None:
-                raise ValueError('No path between nodes!')
+                raise ValueError('No path between nodes {} and {}!'.format(u, v))
         raise ValueError('Iteration limit exceeded!')
 
     @property
@@ -650,7 +655,7 @@ class EnforcedForest(object):
         nodes : set
           Every node currently stored.
         """
-        return set(self.node_data.keys())
+        return self.node_data.keys()
 
     @property
     def children(self):
