@@ -260,18 +260,15 @@ class SceneViewer(pyglet.window.Window):
         # save the rendering mode from the constructor args
         self.vertex_list_mode[name] = args[1]
 
-        try:
-            # if a geometry has UV coordinates that match vertices
-            assert len(geometry.visual.uv) == len(geometry.vertices)
-            has_tex = True
-        except BaseException:
-            has_tex = False
-
-        if has_tex:
-            tex = rendering.material_to_texture(
-                geometry.visual.material)
-            if tex is not None:
-                self.textures[name] = tex
+        # get the visual if the element has it
+        visual = getattr(geometry, 'visual', None)
+        if hasattr(visual, 'uv') and hasattr(visual, 'material'):
+            try:
+                tex = rendering.material_to_texture(visual.material)
+                if tex is not None:
+                    self.textures[name] = tex
+            except BaseException:
+                util.log.warning('failed to load texture', exc_info=True)
 
     def cleanup_geometries(self):
         """
